/* This file is in the public domain */

#include <string>

#include <opencl/filters.h>
#include <opencl/lookup.h>

#include <opencl/emac.h>
#include <opencl/hmac.h>
using namespace OpenCL;

Filter* lookup_mac(const std::string& algname, const SymmetricKey& key)
   {
   Filter* mac = 0;
   try {
     mac = new MAC_Filter(algname, key);
   }
   catch(Algorithm_Not_Found) {}

   if(mac)
      return mac;

   // These are all cases which *should* be handled by lookup
   if(algname == "EMAC<Square>")
      return new MAC_Filter(new EMAC("Square"), key);
   else if(algname == "EMAC<DES>")
      return new MAC_Filter(new EMAC("DES"), key);

   else if(algname == "HMAC-MD2")
      return new MAC_Filter(new HMAC("MD2"), key);
   else if(algname == "HMAC-MD5")
      return new MAC_Filter(new HMAC("MD5"), key);
   else if(algname == "HMAC-SHA-1")
      return new MAC_Filter(new HMAC("SHA-1"), key);
   else if(algname == "HMAC-RIPEMD-128")
      return new MAC_Filter(new HMAC("RIPEMD-128"), key);
   else if(algname == "HMAC-RIPEMD-160")
      return new MAC_Filter(new HMAC("RIPEMD-160"), key);
   else
      return 0;
   }
