/*************************************************
* Diffie-Hellman Header File                     *
* (C) 1999-2002 The OpenCL Project               *
*************************************************/

#ifndef OPENCL_DH_H__
#define OPENCL_DH_H__

#include <opencl/bigint.h>
#include <opencl/pk_keys.h>
#include <opencl/mod_exp.h>

namespace OpenCL {

/*************************************************
* Diffie-Hellman Public Key                      *
*************************************************/
class DH_PublicKey
   {
   public:
      virtual bool check_params() const;
      BigInt get_p() const { return p; }
      BigInt get_g() const { return g; }
      BigInt get_y() const { return y; }

      DH_PublicKey(const BigInt&, const BigInt&, const BigInt&);
      virtual ~DH_PublicKey() {}
   protected:
      DH_PublicKey() {}
      BigInt p, g, y;
   };

/*************************************************
* Diffie-Hellman Private Key                     *
*************************************************/
class DH_PrivateKey : public DH_PublicKey,
                      public PK_Key_Agreement_Key
   {
   public:
      SecureVector<byte> derive_key(const byte[], u32bit) const;
      SecureVector<byte> derive_key(const DH_PublicKey&) const;
      SecureVector<byte> derive_key(const BigInt&) const;

      bool check_params() const;
      BigInt get_x() const { return x; }

      DH_PrivateKey(const BigInt&, const BigInt&, const BigInt& = 0,
                    const BigInt& = 0);
      DH_PrivateKey(u32bit);
   protected:
      FixedExponent_Exp powermod_x_p;
      BigInt x;
   };

}

#endif
